# Every SM has to provide the following methods:  
#   step: (input or None) ->  output
#   currentOutput: () -> output

# Define a primitive state machine by specifying transition, output, and
# initial state functions
# Types:
#    transitionfn: (state, input) -> state
#    outputfn: state -> output
#    initfn: () -> state

verbose = False

class PrimitiveSM:
    def __init__(self, transitionfn, outputfn, initStatefn):
        self.transitionFunction = transitionfn
        self.outputFunction = outputfn
        self.currentState = initStatefn()

    def step(self, input=None):
        oldState = self.currentState
        self.currentState = self.transitionFunction(self.currentState, input)
        if verbose:
            print 'oldState:', oldState, 'input:', input, 'newState:', \
                  self.currentState, 'output:', self.currentOutput()
        return self.currentOutput()

    def currentOutput(self):
        return self.outputFunction(self.currentState)

class SerialSM:
    def __init__(self, sm1, sm2):
        self.m1 = sm1
        self.m2 = sm2

    # Be careful to keep the timing consistent.  The input to m2 has
    # to be the current output of m1, not the output after it is
    # stepped. 
    def step(self, input=None):
        # Your code here
        pass

    def currentOutput(self):
        # Your code here
        pass

class FeedbackSM:
    def __init__(self, sm):
        self.m = sm
    
    def step(self, input=None):
        return self.m.step(self.m.currentOutput())

    def currentOutput(self):
        return self.m.currentOutput()
        
######################################################################
#    Compose a plant and a controller 
######################################################################

def simulatorSM(m1, m2):
    return FeedbackSM(SerialSM(m1, m2))

######################################################################
#    Execution functions
######################################################################

#   Run a state machine that has no input.  Handy for simulators
def run(sm, n = 10):
    result = [sm.currentOutput()]
    print sm.currentOutput()
    for i in range(n):
        print sm.step()
        result.append(sm.currentOutput())
    return result

# Assumes that the list comprehension is calculated in order
def transduce(sm, inputs):
    return [sm.currentOutput()] + [sm.step(item) for item in inputs]
        
######################################################################
#   Examples
######################################################################

## Increment input
def makeIncr(init = 0):
    return PrimitiveSM(lambda s, i: i+1,
                       lambda s: s,
                       lambda : init)

## Delay
def makeDelaySM(init):
    return PrimitiveSM(lambda s, i: i,
                       lambda s: s,
                       lambda : init)

## Sum
def makeSumSM():
    return PrimitiveSM(lambda s, i: s + i,
                       lambda s: s,
                       lambda : 0)

