import SimpleSM
reload(SimpleSM)
from SimpleSM import *

# Every Terminating SM has to provide the following methods:  
#   step: input ->  (output or None)
#                   None only if it is now done
#   currentOutput: () -> output
#   done:  () -> boolean
#   reset: () -> ()


# Define a primitive state machine by specifying transition, output,
# initial state, and termination functions
# Types:
#    transitionfn: (state, input) -> state
#    outputfn: state -> output
#    initStatefn: () -> state 
#    terminationfn: state -> boolean  (defaults to always false)

class PrimitiveTSM(PrimitiveSM):
    def __init__(self, transitionfn, outputfn, initStatefn,
                 terminationfn = lambda x: False):
        self.terminationFunction = terminationfn
        self.initialStateFunction = initStatefn
        PrimitiveSM.__init__(self, transitionfn, outputfn, initStatefn)

    def done(self):
        return self.terminationFunction(self.currentState)

    def reset(self):
        self.currentState = self.initialStateFunction()

class SerialTSM(SerialSM):
    # Your code here
    pass

class FeedbackTSM(FeedbackSM):
    # Your code here
    pass


######################################################################
#    Sequential combinations
######################################################################

class SequentialTSM:
    # Given a list of state machines, execute the first until it is
    # done, then execute the second, etc.
    # Assume they all have the same input space
    
    def __init__(self, smList):
        self.smList = smList
        self.reset()

    def reset(self):
        self.counter = 0
        self.smList[self.counter].reset()

    def step(self, input=None):
        # Your code here
        pass

    def currentOutput(self):
        if not self.done():
            return self.smList[self.counter].currentOutput()

    def done(self):
        # Your code here
        pass

class RepeatTSM:
    # Your code here
    pass

######################################################################
#    Compose a plant and a controller 
######################################################################

def simulatorTSM(m1, m2):
    return FeedbackTSM(SerialTSM(m1, m2))


######################################################################
#   Run a state machine that has no input
######################################################################

def run(sm, n = 100):
    i = 0
    result = [sm.currentOutput()]
    sm.step()
    while not sm.done() and i < n:
        result.append(sm.currentOutput())
        print sm.step()
        i = i + 1
    return result

#############################################################################
##  Other simple examples
#############################################################################

        
#############################################################################
##  Sequencing test
#############################################################################

# Output is the character c
# Transitions to done state after any single input

def makeCharTSM(c):
    # State is True or False (done or not)
    return PrimitiveTSM(lambda s, i: True,     # transition
                        lambda s: c,           # output
                        lambda: False,         # initial state
                        lambda s: s)           # done

# Output a text sequence
def makeTextSequenceTSM(str):
    return SequentialTSM([makeCharTSM(c) for c in str])



